/**
 * P4
 * http://snarfed.org/space/p4
 *
 * Copyright 2001-2004
 * Ken Ashcraft <kashcraft+p4@gmail.com>
 * Ryan Barrett <p4@ryanb.org>
 * Maulik Shah <p4@maulik.net>
 * Nathan Stoll <nathanstoll+p4@gmail.com>
 *
 * @file msg_discover.cpp
 *
 * Defines the cmsg_discover class, which is derived from the abstract cmsg
 * class. The cmsg_discover class is the class that DISCOVER messages
 * instantiate. For more information, see protocol.txt, msg.h, and protocol.h.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "conn_mgr.h"
#include "msg_connect.h"
#include "msg_discover.h"
#include "settings.h"
#include "p4error.h"

#ifdef _WIN32
#include <winsock.h>
#else
#include <netinet/in.h>  // for struct sockaddr_in
#endif

#include <cstdlib>

extern string glocal_user;


/*****************************************************************************
 * INITIALIZE STATIC MEMBERS
 *****************************************************************************/
const float cmsg_discover::kdiscover_relay_prob =
  csettings::instance().get<float>("discover_relay_prob");


/*****************************************************************************
 * PROTECTED METHODS
 *****************************************************************************/

cmsg_discover::cmsg_discover() : cmsg(0, 0, sizeof(p4_discover))
{
  hdr.set_type(kp4_discover);

  data = new char[sizeof(p4_discover)];
  p4_discover *discover = reinterpret_cast<p4_discover *>(data);

  // fill in local username
  strncpy(discover->username, glocal_user.c_str(), kp4_username_len);
  discover->username[kp4_username_len - 1] = '\0';
}


cmsg_discover::cmsg_discover(const p4_header &hdr) : cmsg(hdr)
{
  assert(host_data_len == sizeof(p4_discover) && data);
}


bool cmsg_discover::virt_process() const
{
  float randf = static_cast<float>(rand()) / RAND_MAX;

  if (randf < kdiscover_relay_prob)  {  
    cmsg_discover outgoing(*this);
    
    strcpy(outgoing.hdr.username, glocal_user.c_str());

    // forward message
    // TODO: commented out temporarily because it causes a deadlock
    // 
//    cconn_mgr::instance().random_send(outgoing, hdr.username);

  } else {
    //  cmsg_connect mc;
//    char *remote_user = (reinterpret_cast<p4_discover*>(data))->username;

    // NOTE: we pass FALSE as the lock arg so that conn_mgr doesn't try to
    // acquire its mutex - it already has it, since process is called from
    // cconn_mgr::receive
    // 
    // TODO: commented out temporarily because it causes a deadlock
    // 
/*    if (mc.send(remote_user, false).get_status() == cconn::efailed) {
      cdebug::instance() << acquire("msg_debug") <<
        "msg_discover::virt_process(). could not respond to " <<
        remote_user << release();
    }    
*/  }
  return true;
}


/***************************************************************************
 * TEST MAIN
 **************************************************************************/
#ifdef CMSG_DISCOVER_MAIN
int main(int argc, char **argv)
{
  cmsg_discover d;

  cout << d;

  return 0;
}
#endif
