/**
 * P4
 * http://snarfed.org/space/p4
 *
 * Copyright 2001-2004
 * Ken Ashcraft <kashcraft+p4@gmail.com>
 * Ryan Barrett <p4@ryanb.org>
 * Maulik Shah <p4@maulik.net>
 * Nathan Stoll <nathanstoll+p4@gmail.com>
 *
 * @file rwlock.cpp
 *
 * Pthreads on linux didn't have support for the rwlock stuff from the pthreads
 * standard until recently. So, while it didn't, we used these wrappers for a
 * basic semaphore. (That meant that performance was better on non-linux
 * platforms.)
 *
 * Now, of course, we don't need to use this any more.
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

// make sure this file is not used!
#error "PThreads on Linux supports rwlocks now. Don't use ours!"

#include "rwlock.h"
#include "settings.h"

static int kmax_readers = 10;
  // this doesn't work...?
  // = csettings::instance().get<int>("lka_max_readers");

int pthread_rwlock_init(pthread_rwlock_t *rwlock, 
                        const pthread_rwlockattr_t *attr)
{
  sem_init(&(rwlock->write_lock), 0, 1);
  return sem_init(&(rwlock->read_lock), 0, kmax_readers);
}

int pthread_rwlock_destroy(pthread_rwlock_t *rwlock)
{
  sem_destroy(&(rwlock->write_lock));
  return sem_destroy(&(rwlock->read_lock));
}

int pthread_rwlock_rdlock(pthread_rwlock_t *rwlock)
{
  return sem_wait(&(rwlock->read_lock));
}

int pthread_rwlock_wrlock(pthread_rwlock_t *rwlock)
{
  int ret;
  
  sem_wait(&(rwlock->write_lock));
  for (int i = 0; i < kmax_readers; i++)
    ret = sem_wait(&(rwlock->read_lock));

  return ret;
}

int pthread_rwlock_unlock(pthread_rwlock_t *rwlock)
{
  int writer_val;

  sem_getvalue(&(rwlock->write_lock), &writer_val);

  if (writer_val <= 0) { // we're the writer
    for (int i = 0; i < kmax_readers; i++)
      sem_post(&(rwlock->read_lock));

    return sem_post(&(rwlock->write_lock));
  }

  // we're just a reader
  return sem_post(&(rwlock->read_lock));
}

